/*
 * The Clear BSD License
 * Copyright (c) 2016, Freescale Semiconductor, Inc.
 * Copyright 2016-2017 NXP
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without modification,
 * are permitted (subject to the limitations in the disclaimer below) provided
 * that the following conditions are met:
 *
 * o Redistributions of source code must retain the above copyright notice, this list
 *   of conditions and the following disclaimer.
 *
 * o Redistributions in binary form must reproduce the above copyright notice, this
 *   list of conditions and the following disclaimer in the documentation and/or
 *   other materials provided with the distribution.
 *
 * o Neither the name of Freescale Semiconductor, Inc. nor the names of its
 *   contributors may be used to endorse or promote products derived from this
 *   software without specific prior written permission.
 *
 * NO EXPRESS OR IMPLIED LICENSES TO ANY PARTY'S PATENT RIGHTS ARE GRANTED BY THIS LICENSE.
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR
 * ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
 * ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */


#ifndef _PIN_MUX_H_
#define _PIN_MUX_H_

/***********************************************************************************************************************
 * Definitions
 **********************************************************************************************************************/

/*! @brief Direction type  */
typedef enum _pin_mux_direction
{
    kPIN_MUX_DirectionInput = 0U,        /* Input direction */
    kPIN_MUX_DirectionOutput = 1U,       /* Output direction */
    kPIN_MUX_DirectionInputOrOutput = 2U /* Input or output direction */
} pin_mux_direction_t;

/*!
 * @addtogroup pin_mux
 * @{
 */

/***********************************************************************************************************************
 * API
 **********************************************************************************************************************/

#if defined(__cplusplus)
extern "C" {
#endif


/*! @name PORTC6 (number 81), UART1_RX_TGTMCU
  @{ */
#define BOARD_DEBUG_UART_RX_PERIPHERAL LPUART1      /*!<@brief Device name: LPUART1 */
#define BOARD_DEBUG_UART_RX_SIGNAL RX               /*!<@brief LPUART1 signal: RX */
#define BOARD_DEBUG_UART_RX_PORT PORTC              /*!<@brief PORT device name: PORTC */
#define BOARD_DEBUG_UART_RX_PIN 6U                  /*!<@brief PORTC pin index: 6 */
#define BOARD_DEBUG_UART_RX_PIN_NAME LPUART1_RX     /*!<@brief Pin name */
#define BOARD_DEBUG_UART_RX_LABEL "UART1_RX_TGTMCU" /*!<@brief Label */
#define BOARD_DEBUG_UART_RX_NAME "DEBUG_UART_RX"    /*!<@brief Identifier name */
                                                    /* @} */

/*! @name PORTC7 (number 80), UART1_TX_TGTMCU
  @{ */
#define BOARD_DEBUG_UART_TX_PERIPHERAL LPUART1      /*!<@brief Device name: LPUART1 */
#define BOARD_DEBUG_UART_TX_SIGNAL TX               /*!<@brief LPUART1 signal: TX */
#define BOARD_DEBUG_UART_TX_PORT PORTC              /*!<@brief PORT device name: PORTC */
#define BOARD_DEBUG_UART_TX_PIN 7U                  /*!<@brief PORTC pin index: 7 */
#define BOARD_DEBUG_UART_TX_PIN_NAME LPUART1_TX     /*!<@brief Pin name */
#define BOARD_DEBUG_UART_TX_LABEL "UART1_TX_TGTMCU" /*!<@brief Label */
#define BOARD_DEBUG_UART_TX_NAME "DEBUG_UART_TX"    /*!<@brief Identifier name */
                                                    /* @} */

/*! @name PORTD0 (number 4), D4[1]/RGB_RED
  @{ */
#define BOARD_LED_RED_PERIPHERAL FTM0       /*!<@brief Device name: FTM0 */
#define BOARD_LED_RED_SIGNAL CH             /*!<@brief FTM0 signal: CH */
#define BOARD_LED_RED_PORT PORTD            /*!<@brief PORT device name: PORTD */
#define BOARD_LED_RED_PIN 0U                /*!<@brief PORTD pin index: 0 */
#define BOARD_LED_RED_CHANNEL 2             /*!<@brief FTM0 channel: 2 */
#define BOARD_LED_RED_PIN_NAME FTM0_CH2     /*!<@brief Pin name */
#define BOARD_LED_RED_LABEL "D4[1]/RGB_RED" /*!<@brief Label */
#define BOARD_LED_RED_NAME "LED_RED"        /*!<@brief Identifier name */
                                            /* @} */

/*! @name PORTD15 (number 22), D4[3]/RGB_BLUE
  @{ */
#define BOARD_LED_BLUE_PERIPHERAL FTM0        /*!<@brief Device name: FTM0 */
#define BOARD_LED_BLUE_SIGNAL CH              /*!<@brief FTM0 signal: CH */
#define BOARD_LED_BLUE_PORT PORTD             /*!<@brief PORT device name: PORTD */
#define BOARD_LED_BLUE_PIN 15U                /*!<@brief PORTD pin index: 15 */
#define BOARD_LED_BLUE_CHANNEL 0              /*!<@brief FTM0 channel: 0 */
#define BOARD_LED_BLUE_PIN_NAME FTM0_CH0      /*!<@brief Pin name */
#define BOARD_LED_BLUE_LABEL "D4[3]/RGB_BLUE" /*!<@brief Label */
#define BOARD_LED_BLUE_NAME "LED_BLUE"        /*!<@brief Identifier name */
                                              /* @} */

/*! @name PORTB9 (number 76), U1[3]/FXOS_RST
  @{ */
#define BOARD_ACCEL_RST_PERIPHERAL GPIOB       /*!<@brief Device name: GPIOB */
#define BOARD_ACCEL_RST_SIGNAL GPIO            /*!<@brief GPIOB signal: GPIO */
#define BOARD_ACCEL_RST_GPIO GPIOB             /*!<@brief GPIO device name: GPIOB */
#define BOARD_ACCEL_RST_GPIO_PIN 9U            /*!<@brief PORTB pin index: 9 */
#define BOARD_ACCEL_RST_PORT PORTB             /*!<@brief PORT device name: PORTB */
#define BOARD_ACCEL_RST_PIN 9U                 /*!<@brief PORTB pin index: 9 */
#define BOARD_ACCEL_RST_CHANNEL 9              /*!<@brief GPIOB GPIO channel: 9 */
#define BOARD_ACCEL_RST_PIN_NAME PTB9          /*!<@brief Pin name */
#define BOARD_ACCEL_RST_LABEL "U1[3]/FXOS_RST" /*!<@brief Label */
#define BOARD_ACCEL_RST_NAME "ACCEL_RST"       /*!<@brief Identifier name */
                                               /* @} */

/*!
 * @brief Configures pin routing and optionally pin electrical features.
 *
 */
void BOARD_InitPins(void);

/*! @name PORTA3 (number 72), J4[12]/Arduino_A5_ADC1_SE1/FXOS_SCL
  @{ */
/*!
 * @brief Device name: LPI2C0 */
#define BOARD_I2C_CONFIGUREPINS_ACCEL_I2C_SCL_PERIPHERAL LPI2C0
/*!
 * @brief LPI2C0 signal: SCL */
#define BOARD_I2C_CONFIGUREPINS_ACCEL_I2C_SCL_SIGNAL SCL
/*!
 * @brief PORT device name: PORTA */
#define BOARD_I2C_CONFIGUREPINS_ACCEL_I2C_SCL_PORT PORTA
/*!
 * @brief PORTA pin index: 3 */
#define BOARD_I2C_CONFIGUREPINS_ACCEL_I2C_SCL_PIN 3U
/*!
 * @brief Pin name */
#define BOARD_I2C_CONFIGUREPINS_ACCEL_I2C_SCL_PIN_NAME LPI2C0_SCL
/*!
 * @brief Label */
#define BOARD_I2C_CONFIGUREPINS_ACCEL_I2C_SCL_LABEL "J4[12]/Arduino_A5_ADC1_SE1/FXOS_SCL"
/*!
 * @brief Identifier name */
#define BOARD_I2C_CONFIGUREPINS_ACCEL_I2C_SCL_NAME "ACCEL_I2C_SCL"
/* @} */

/*! @name PORTA2 (number 73), J4[10]/Arduino_A4_ADC1_SE0/FXOS_SDA
  @{ */
/*!
 * @brief Device name: LPI2C0 */
#define BOARD_I2C_CONFIGUREPINS_ACCEL_I2C_SDA_PERIPHERAL LPI2C0
/*!
 * @brief LPI2C0 signal: SDA */
#define BOARD_I2C_CONFIGUREPINS_ACCEL_I2C_SDA_SIGNAL SDA
/*!
 * @brief PORT device name: PORTA */
#define BOARD_I2C_CONFIGUREPINS_ACCEL_I2C_SDA_PORT PORTA
/*!
 * @brief PORTA pin index: 2 */
#define BOARD_I2C_CONFIGUREPINS_ACCEL_I2C_SDA_PIN 2U
/*!
 * @brief Pin name */
#define BOARD_I2C_CONFIGUREPINS_ACCEL_I2C_SDA_PIN_NAME LPI2C0_SDA
/*!
 * @brief Label */
#define BOARD_I2C_CONFIGUREPINS_ACCEL_I2C_SDA_LABEL "J4[10]/Arduino_A4_ADC1_SE0/FXOS_SDA"
/*!
 * @brief Identifier name */
#define BOARD_I2C_CONFIGUREPINS_ACCEL_I2C_SDA_NAME "ACCEL_I2C_SDA"
/* @} */

/*!
 * @brief Configures pin routing and optionally pin electrical features.
 *
 */
void BOARD_I2C_ConfigurePins(void);

#if defined(__cplusplus)
}
#endif

/*!
 * @}
 */
#endif /* _PIN_MUX_H_ */

/***********************************************************************************************************************
 * EOF
 **********************************************************************************************************************/
