/*
 * The Clear BSD License
 * Copyright (c) 2016, Freescale Semiconductor, Inc.
 * Copyright 2016-2017 NXP
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without modification,
 * are permitted (subject to the limitations in the disclaimer below) provided
 * that the following conditions are met:
 *
 * o Redistributions of source code must retain the above copyright notice, this list
 *   of conditions and the following disclaimer.
 *
 * o Redistributions in binary form must reproduce the above copyright notice, this
 *   list of conditions and the following disclaimer in the documentation and/or
 *   other materials provided with the distribution.
 *
 * o Neither the name of the copyright holder nor the names of its
 *   contributors may be used to endorse or promote products derived from this
 *   software without specific prior written permission.
 *
 * NO EXPRESS OR IMPLIED LICENSES TO ANY PARTY'S PATENT RIGHTS ARE GRANTED BY THIS LICENSE.
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR
 * ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
 * ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#include "fsl_common.h"
#include "fsl_debug_console.h"
#include "board.h"

#include "fsl_tsi_v5.h"
#include "TSI_key.h"

#include "pin_mux.h"
/*******************************************************************************
 * Definitions
 ******************************************************************************/
#define LED1_INIT() LED_RED1_INIT(LOGIC_LED_OFF)
#define LED1_ON() LED_RED1_ON()
#define LED1_OFF() LED_RED1_OFF()

/*******************************************************************************
 * Variables
 ******************************************************************************/

/*******************************************************************************
 * Prototypes
 ******************************************************************************/

/*******************************************************************************
 * Code
 ******************************************************************************/

/*!
 * @brief Initiate TSI module
 * This function is used to initiate TSI moudle,set charge current/voltage,
 * oscillator frequency, track TSI baseline through TSI calibration function.
 */
void TSI_init(void)
{
    tsi_selfCap_config_t config;

    TSI_GetSelfCapModeDefaultConfig(&config);
    config.commonConfig.mode = kTSI_SensingModeSlection_Self;
    config.commonConfig.mainClock = kTSI_MainClockSlection_1;
    config.commonConfig.dvolt = kTSI_DvoltOption_0;               /* DVOLT option value  */
    config.commonConfig.cutoff = kTSI_SincCutoffDiv_0;            /* Cutoff divider  */
    config.commonConfig.order = kTSI_SincFilterOrder_2;           /* SINC filter order */
    config.commonConfig.decimation = kTSI_SincDecimationValue_8;  /* SINC decimation value */
    config.commonConfig.chargeNum = kTSI_SscChargeNumValue_7;     /* SSC output bit0's period setting */
    config.commonConfig.noChargeNum = kTSI_SscNoChargeNumValue_5; /* SSC output bit1's period setting */

    config.enableSensitivity = 0;                             /* Enable sensitivity boost of self-cap or not */
    config.xdn = kTSI_SensitivityXdnOption_1;                 /* Sensitivity XDN option   */
    config.ctrim = kTSI_SensitivityCtrimOption_7;             /* Sensitivity CTRIM option */
    config.inputCurrent = kTSI_CurrentMultipleInputValue_0;   /* Input current multiple   */
    config.chargeCurrent = kTSI_CurrentMultipleChargeValue_1; /* Charge/Discharge current multiple */

    TSI_InitSelfCapMode(TSI, &config);

    TSI_EnableModule(TSI, true);
}

/*!
 * @brief Main function
 */
int main(void)
{
    uint8_t key_event;
    uint8_t touched_key_id = 0xFFU;

    /* Init board hardware. */
    BOARD_InitPins();
    BOARD_BootClockRUN();
    BOARD_InitDebugConsole();

    PRINTF("TSI self mode demo.\r\n");

    LED1_INIT();
    /* Init TSI moudle*/
    TSI_init();
    /* Calibrate TSI idle value, save as TSI baseline */
    TSI_keyCalibrate();

    while (1)
    {
        /* debug key function */
        key_event = TSI_keyDetect(&touched_key_id);

        if (key_event == kKey_Event_Touch)
        {
            /* if (touched_key_id == 0) */
            {
                LED1_ON();
                PRINTF("TSI key touched.\r\n");
            }
        }
        else if (key_event == kKey_Event_Release)
        {
            /* if (touched_key_id == 0) */
            {
                LED1_OFF();
                PRINTF("TSI key released.\r\n");
            }
        }
        else
        {
            /* Here to deal with other key events */
            key_event = kKey_Event_Idle;
        }
    }
}
